use std::sync::Arc;

use vulkano::{
    command_buffer::allocator::StandardCommandBufferAllocator,
    descriptor_set::allocator::StandardDescriptorSetAllocator,
    device::{Device, Queue},
    format::Format,
    instance::Instance,
    memory::allocator::StandardMemoryAllocator,
};

pub struct RenderContext {
    pub(super) instance: Arc<Instance>,
    pub(super) device: Arc<Device>,
    pub(super) graphics_queue: Arc<Queue>,
    pub(super) compute_queue: Arc<Queue>,
    pub(super) transfer_queue: Option<Arc<Queue>>,
    pub(super) memory_allocator: Arc<StandardMemoryAllocator>,
    pub(super) command_buffer_allocator: Arc<StandardCommandBufferAllocator>,
    pub(super) descriptor_set_allocator: Arc<StandardDescriptorSetAllocator>,
    pub(super) window_size: [f32; 2],
    pub(super) aspect_ratio: f32,
    pub(super) swapchain_format: Format,
}

impl RenderContext {
    pub fn instance(&self) -> &Arc<Instance> {
        &self.instance
    }

    pub fn device(&self) -> &Arc<Device> {
        &self.device
    }

    pub fn graphics_queue(&self) -> &Arc<Queue> {
        &self.graphics_queue
    }

    pub fn compute_queue(&self) -> &Arc<Queue> {
        &self.compute_queue
    }

    pub fn transfer_queue(&self) -> Option<&Arc<Queue>> {
        self.transfer_queue.as_ref()
    }

    pub fn memory_allocator(&self) -> &Arc<StandardMemoryAllocator> {
        &self.memory_allocator
    }

    pub fn command_buffer_allocator(&self) -> &Arc<StandardCommandBufferAllocator> {
        &self.command_buffer_allocator
    }

    pub fn descriptor_set_allocator(&self) -> &Arc<StandardDescriptorSetAllocator> {
        &self.descriptor_set_allocator
    }

    pub fn window_size(&self) -> &[f32; 2] {
        &self.window_size
    }

    pub fn aspect_ratio(&self) -> f32 {
        self.aspect_ratio
    }

    pub fn swapchain_format(&self) -> Format {
        self.swapchain_format
    }
}
